//
//      SGXAWSParser.cpp
//
//      This source code was automatically
//      generated by SophiaFramework 4.1.1
//

#include "pself3.hpp"
#include "AWSListProducts.hpp"

// Constructor
AWSListProducts::AWSListProducts(Void) 
{
    return;
}

// Destructor
AWSListProducts::~AWSListProducts(Void)
{
	_errmsg.Clear();
	
	SFXArray<AWSProductPtr>::Enumerator enumerator = _productarray.GetFirstEnumerator();
    while (enumerator.HasNext()) 
	{
		AWSProductPtr item(enumerator.GetNext());
		delete item;
    }
    _productarray.Clear();

    return;
}

// get i-th product
AWSProductPtr AWSListProducts::GetProduct(UInt16 index)
{
	if (index < _productarray.GetSize()) {
        return _productarray[index];
    }
    return null;
}

// get list of products from favorite file

SFCError AWSListProducts::GetFavoriteProducts(SFXPathConstRef filename, SFXAnsiString producttype)
{
	SFCError error;
	UInt16 i, size;

	SFXConfig _config;
 	AWSProductPtr item;

    // Load item list from file
    error = _config.Load(filename);
	if (error != SFERR_NO_ERROR)
	{
		_errmsg = "Cannot read from file";
		return error;
	}

    // get number of items
    size = _config.ReadUInt16(0, 0);

    for (i = 0; i < size; i++) {
		//process each item
		item = AWSProduct::Factory(producttype);
		item->SetAsin(_config.ReadSFXAnsiString(i * 2 + 1, ""));
		item->SetTitle(_config.ReadSFXAnsiString(i * 2 + 2, ""));
		
		error = _productarray.Append(item);
		if (error != SFERR_NO_ERROR) break;
    }

	return error;
}

// update favorite file
Void AWSListProducts::UpdateFavoriteFile(SFXPathConstRef filename)
{
 	SFXConfig _config;
	UInt16 i, size;

	size = (UInt16) _productarray.GetSize();
	if (size<=0)
	{
		SFXFile::Remove(filename);
		return;
	}

    _config.Load(filename);
	
	// update number of favorite items 
    _config.WriteUInt16(0, size);

	// update product list to favorite file 
    for (i = 0; i < size; i++) {
        _config.WriteSFXAnsiString(i * 2 + 1, _productarray[i]->GetAsin());
        _config.WriteSFXAnsiString(i * 2 + 2, _productarray[i]->GetTitle());
    }
    _config.Save();
}

// remove a product from list and delete related files
SFCError AWSListProducts::RemoveProduct(SFXAnsiStringConstRef asin)
{
    SInt16 i;
	SFCError error(SFERR_NO_ERROR);
	SFXPathConst filenameXML(DIR_NAME + asin+ ".xml");
	SFXPathConst filenameJPG(DIR_NAME + asin + ".jpg");

	for (i = 0; i < _productarray.GetSize(); i++) 
	{
		if (_productarray[i]->GetAsin() == asin) 
		{
			SFXFile::Remove(filenameXML);
			SFXFile::Remove(filenameJPG);
			delete _productarray[i];
            _productarray.Remove(i);
            
			--i;
            error = SFERR_NO_ERROR;
			break;
        }
    }
    return error;
}

// remove all products from list and delete all related files
SFCError AWSListProducts::RemoveAllProducts()
{
    SFCError error(SFERR_NO_ERROR);
	SFXPath filenameXML, filenameJPG;

	SFXArray<AWSProductPtr>::Enumerator enumerator = _productarray.GetFirstEnumerator();
	while (enumerator.HasNext()) 
	{
		AWSProductPtr item(enumerator.GetNext());
		filenameXML = SFXPath(DIR_NAME + item->GetAsin() + ".xml");
		filenameJPG = SFXPath(DIR_NAME + item->GetAsin() + ".jpg");
		SFXFile::Remove(filenameXML);
		SFXFile::Remove(filenameJPG);
        delete item;
    }
    _productarray.Clear();
	return error;
}

//Parse a XML file to get products
SFCError AWSListProducts::GetXMLProducts(SFXPathConstRef filename, SFXAnsiString producttype)
{
	SFCError error;
	SInt32 size;
	SInt32 i;

	SFXXMLDOMParser parser; // parse XML document by SFXXMLDOMParser
	AWSProductPtr item;		// pointer to AWSProduct abtract class

	parser.SetDoNamespaces(false);

	// for debug purpose
	SFXHelper::dbgprintf("[SGXAWSParser] [%s]", filename.AsFile().GetCString());

	// parse XML file using DOM parser
	// this DOM parser expands the XML document onto memory as a tree
	error = parser.Parse(filename); 
	SFXHelper::dbgprintf("[SGXAWSParser.Parse] [%d]", error);

	// if parsing failed
	if (error != SFERR_NO_ERROR)
	{
		_errmsg = "Parsing failed";
		return error;
	}

	// get the document root
	SFXXMLDocumentPtr root = parser.GetDocument(); 
    
	// if no root found
	if (root==null)
	{
		error = SFERR_FAILED;
		_errmsg = "No data found";
		return error;
	}
    
	SFXXMLNodePtr _anode;

	// Get list of Items
	SFXList<SFXXMLNodePtr>* list = root->GetElementsByTagName("Item");
	
	// if no Item found
	if (list == null || list->GetSize() == 0)
	{
		error = SFERR_FAILED;
		_errmsg = "No item found";
		return error;
	}
    
	size = list->GetSize();
	
	for (i=0; i<size; i++)
	{// process each item
		_anode = list->Get(i); 
		// item now is the pointer to actual product class
		item = AWSProduct::Factory(producttype);
		item->SetParseImageUrl(true);
        item->SetParseCustomerReviews(true);
        item->SetParseSimilarProducts(true);
		
		// parse this node and store information in the item
		error = ParseItem(_anode, item);
		
		if (error == SFERR_NO_ERROR)
		{
			// put the item to the product list
			error = _productarray.Append(item);
			// if error then stop
			if (error != SFERR_NO_ERROR) break; 
		}	
		
	}
   
	if (error != SFERR_NO_ERROR) 
		_errmsg = "Parsing failed";
    
	return error;
}

//Parse one item
SFCError AWSListProducts::ParseItem(SFXXMLNodePtr node, AWSProductPtr item)
{
	SFCError error(SFERR_NO_ERROR);
	
	SFXAnsiString label;		// node label
	SFXXMLNodePtr child(node);	// begin from first child

	//
	// process this node to get information then store in the item
	//
	for (child = child->GetFirstChild(); child != null && error == SFERR_NO_ERROR; child = child->GetNextSibling())
	{
		label = SFXAnsiString(child->GetNodeName());
		
		// get asin value
		if (label == "ASIN")
			item->SetAsin(item->DecodeHtml(child->GetText()));
		// get detail URL
		else if (label == "DetailPageURL")
			item->SetDetailPageURL(item->DecodeHtml(child->GetText()));
		// get sail rank
		else if (label == "SalesRank")
			item->SetSalesRank(child->GetText().Replace(',', "").AsSInt32());
		// get item attributes
		else if (label == "ItemAttributes")
		{
			if (item->IsParseItemAttributes()) 
				error = item->ParseItemAttributes(child->GetFirstChild());
			else 
				error = item->ParseTitle(child->GetFirstChild());
		}
		// get small image
		else if (label == "SmallImage")
		{
			if (item->IsParseImageUrl()) 
				error = item->ParseSmallImage(child->GetFirstChild());
		}    
		// get medium image
		else if (label == "MediumImage")
		{
			if (item->IsParseImageUrl())
				error = item->ParseMediumImage(child->GetFirstChild());
		}
		// get large image
		else if (label == "LargeImage")
		{
			if (item->IsParseImageUrl())
				error = item->ParseLargeImage(child->GetFirstChild());
		}     
		// get Offer Summary
		else if (label == "OfferSummary")
		{
			if (item->IsParseOfferSummary())
				error = item->ParseOfferSummary(child->GetFirstChild());
		}           
		// get Offer
		else if (label == "Offers")
		{
			if (item->IsParseOffers())
				error = item->ParseOffers(child->GetFirstChild());
		}       
		// get Customer Reviews
		else if (label == "CustomerReviews")
		{
			if (item->IsParseCustomerReviews())
				error = item->ParseCustomerReviews(child->GetFirstChild());
		}
		// get Similar Products
		else if (label == "SimilarProducts")
		{
			if (item->IsParseSimilarProducts()) 
				error = item->ParseSimilarProducts(child->GetFirstChild());
		}
		// get Browse Nodes
		else if (label == "BrowseNodes")
		{
			if (item->IsParseBrowserNodes()) 
				error = item->ParseBrowserNodes(child->GetFirstChild());
		}
		// get Listmania Lists
		else if (label == "ListmaniaLists")
		{
			if (item->IsParseListmaniaLists())
				error = item->ParseListmaniaLists(child->GetFirstChild());	
		}
	}
	return error;
}


